#-*- coding:utf-8 -*-

import os, datetime, cgi, StringIO
from wsgiref import util, simple_server
from xml.sax import saxutils

import model


class MessageBoard(object):
    ''' メッセージボードサービスを提供するアプリケーション '''

    def __init__(self):
        ''' 初期化 '''


    def __call__(self, environ, start_response):
        ''' WSGI アプリケーション '''

        # リクエストメソッドを取得
        method = environ['REQUEST_METHOD']

        if method == 'GET':
            # GET の場合
            return self.listMessages(environ, start_response)

        elif method == 'POST':
            # POST の場合
            return self.addMessage(environ, start_response)

        else:
            # それ以外の場合は 501 を返す
            start_response('501 NotImplemented', [('Content-type', 'text/plain')])
            return 'Not Implemented'


    def listMessages(self, environ, start_response):
        ''' 一覧表示 '''

        fp = StringIO.StringIO()

        # ヘッダを出力
        fp.write(r'''<html>
<head><title>Message Board</title>
<meta http-equiv="Content-type" content="text/html; charset=utf-8">
</head>
<body>
''')

        # メッセージ数分繰り返し
        for msg in model.getEntries():

            esc = saxutils.escape

            msg = {'name':msg.name,
                   'title':msg.title,
                   'body':msg.body,
                   'date':msg.date}

            tmp = {}
            
            # 入力を全てエスケープして出力
            for key, value in msg.iteritems():
                t = unicode(value)
                t = esc(t)
                tmp[key] = t.encode('utf-8')


            # メッセージの内容を書き出す
            fp.write('''<dl>
<dt>name</dt>
<dd>%(name)s</dd>
<dt>title</dt>
<dd>%(title)s</dd>
<dt>date</dt>
<dd>%(date)s</dd>
<dt>message</dt>
<dd>%(body)s</dd>
</dl><hr />''' % tmp)

        # 書込み用フォームを出力
        fp.write('''<form action="%s" method="POST" 
AcceotEncoding="utf-8">
<dl>
<dt>name</dt>
<dd><input name="name" type="text" /></dd>
<dt>title</dt>
<dd><input type="text" name="title"/></dd>
<dt>body</dt>
<dd><textarea name="body"></textarea></dd>
</dl>
<input type="submit" name="save" value="Post" />
</form>
</body></html>''' % (util.request_uri(environ, False),))

        # シーク位置を先頭にしておく
        fp.seek(0)

        start_response('200 OK', [('Content-type', 'text/html; charset=utf-8')])
        return fp


    def addMessage(self, environ, start_response):
        ''' メッセージを追加する '''

        # POST データを取得
        inpt = environ['wsgi.input']
        length = int(environ.get('CONTENT_LENGTH', 0))

        # 取得したデータをパースして辞書オブジェクトに変換
        query = dict(cgi.parse_qsl(inpt.read(length)))

        model.addEntry(name=unicode(query['name'], 'utf-8', 'ignore'),
                       title=unicode(query['title'], 'utf-8', 'ignore'),
                       body=unicode(query['body'], 'utf-8', 'ignore'))

        # リダイレクトを行う
        start_response('303 See Other', [('Content-type', 'text/plain'),
                                         ('Location', util.request_uri(environ))])

        return ''



def main():

    from wsgiref import handlers

    application = MessageBoard()
    
    handlers.CGIHandler().run(application)



if __name__ == '__main__':

    main()

